﻿using System;
using System.Activities;
using System.Collections.ObjectModel;
using System.Text;

using Microsoft.Xrm.Sdk;
using Microsoft.Xrm.Sdk.Query;
using Microsoft.Xrm.Sdk.Workflow;

namespace Dyn.Xrm.Workflows
{
    public sealed class AddressOnCreateUpdate : CodeActivity
    {
        protected override void Execute(CodeActivityContext executionContext)
        {
            //Create the context
            IWorkflowContext context = executionContext.GetExtension<IWorkflowContext>();
            IOrganizationServiceFactory serviceFactory = executionContext.GetExtension<IOrganizationServiceFactory>();
            IOrganizationService service = serviceFactory.CreateOrganizationService(context.UserId);

            string street1 = Street1.Get<string>(executionContext);
            string street2 = Street2.Get<string>(executionContext);
            string street3 = Street3.Get<string>(executionContext);
            string city = City.Get<string>(executionContext);
            string state = State.Get<string>(executionContext);
            string postalCode = PostalCode.Get<string>(executionContext);

            string compositeAddress = GenerateFullAddress(street1, street2, street3, city, state, postalCode);
            Address.Set(executionContext, compositeAddress);

        }

        /// <summary>
        /// Generate Full Address Format
        /// Street 1
        /// Street 2
        /// City, State Zip
        /// </summary>
        private string GenerateFullAddress(string line1, string line2, string line3, string city, string state, string postalcode)
        {
            StringBuilder sb = new StringBuilder();

            if (!String.IsNullOrEmpty(line1))
            {
                sb.Append(line1);
                sb.AppendLine();
            }

            if (!String.IsNullOrEmpty(line2))
                sb.AppendLine(line2);

            if (!String.IsNullOrEmpty(city))
            {
                sb.Append(city);
                if (!String.IsNullOrEmpty(state))
                {
                    sb.Append(", " + state);
                    if (!String.IsNullOrEmpty(postalcode))
                    {
                        sb.Append(" " + postalcode);
                    }
                }
                else
                {
                    if (!String.IsNullOrEmpty(postalcode))
                    {
                        sb.Append(" " + postalcode);
                    }
                }
            }
            else
            {
                if (!String.IsNullOrEmpty(state))
                {
                    sb.Append(", " + state);
                    if (!String.IsNullOrEmpty(postalcode))
                    {
                        sb.Append(" " + postalcode);
                    }
                }
                else
                {
                    if (!String.IsNullOrEmpty(postalcode))
                    {
                        sb.Append(" " + postalcode);
                    }
                }
            }
            return sb.ToString();
        }


        [RequiredArgument]
        [Input("Street 1")]
        public InArgument<string> Street1 { get; set; }

        [RequiredArgument]
        [Input("Street 2")]
        public InArgument<string> Street2 { get; set; }

        [Input("Street 3")]
        public InArgument<string> Street3 { get; set; }

        [RequiredArgument]
        [Input("City")]
        public InArgument<string> City { get; set; }

        [RequiredArgument]
        [Input("State")]
        public InArgument<string> State { get; set; }

        [RequiredArgument]
        [Input("Postal Code")]
        public InArgument<string> PostalCode { get; set; }

        [Output("Composite Address")]
        public OutArgument<string> Address { get; set; }

    }
}
